@@ -36,7 +36,7 @@
 #define THEME_ITEM_NORMAL()           do {                            } while(0)
 #define THEME_ITEM_FOCUS()            do {                            } while(0)
 #define THEME_ITEM_BOLD()             do { attrmode(ATTR_BOLD_ON);    } while(0)
-#define THEME_ITEM_SELECTED()         do { attrmode(ATTR_REVERSE_ON); } while(0)
+#define THEME_ITEM_SELECTED()         do { if (p->focused) attrmode(ATTR_REVERSE_ON); } while(0)
 #define THEME_SCROLLBAR_FOCUS()       do {                            } while(0)
 #define THEME_SCROLLBAR_NORMAL()      do { attrmode(ATTR_FAINT_ON);   } while(0)
 #define THEME_SCROLLBAR_TICK_FOCUS()  do { attrmode(ATTR_REVERSE_ON); } while(0)
@@ -50,6 +50,9 @@ static char *pipercmd = "sfeed_content"; /* env variable: $SFEED_PIPER */
 static char *yankercmd = "xclip -r"; /* env variable: $SFEED_YANKER */
 static char *markreadcmd = "sfeed_markread read"; /* env variable: $SFEED_MARK_READ */
 static char *markunreadcmd = "sfeed_markread unread"; /* env variable: $SFEED_MARK_UNREAD */
+static int plumberia = 0; /* env variable: $SFEED_PLUMBER_INTERACTIVE */
+static int piperia = 1; /* env variable: $SFEED_PIPER_INTERACTIVE */
+static int yankeria = 0; /* env variable: $SFEED_YANKER_INTERACTIVE */
 
 enum {
 	ATTR_RESET = 0,	ATTR_BOLD_ON = 1, ATTR_FAINT_ON = 2, ATTR_REVERSE_ON = 7
@@ -59,7 +62,8 @@ enum Pane { PaneFeeds, PaneItems, PaneLast };
 
 enum {
 	FieldUnixTimestamp = 0, FieldTitle, FieldLink, FieldContent,
-	FieldContentType, FieldId, FieldAuthor, FieldEnclosure, FieldLast
+	FieldContentType, FieldId, FieldAuthor, FieldEnclosure,
+	FieldCategory, FieldLast
 };
 
 struct win {
@@ -140,11 +144,12 @@ struct feed {
 void alldirty(void);
 void cleanup(void);
 void draw(void);
+int getsidebarwidth(void);
 void markread(struct pane *, off_t, off_t, int);
 void pane_draw(struct pane *);
 void sighandler(int);
 void updategeom(void);
-void updatesidebar(int);
+void updatesidebar(void);
 void urls_free(void);
 int urls_isnew(const char *);
 void urls_read(void);
@@ -493,6 +498,7 @@ cleanup(void)
 
 	resettitle();
 
+	memset(&sa, 0, sizeof(sa));
 	sigemptyset(&sa.sa_mask);
 	sa.sa_flags = SA_RESTART; /* require BSD signal semantics */
 	sa.sa_handler = SIG_DFL;
@@ -543,6 +549,7 @@ init(void)
 
 	updategeom();
 
+	memset(&sa, 0, sizeof(sa));
 	sigemptyset(&sa.sa_mask);
 	sa.sa_flags = SA_RESTART; /* require BSD signal semantics */
 	sa.sa_handler = sighandler;
@@ -554,24 +561,50 @@ init(void)
 	needcleanup = 1;
 }
 
+void
+processexit(pid_t pid, int interactive)
+{
+	pid_t wpid;
+	struct sigaction sa;
+
+	memset(&sa, 0, sizeof(sa));
+	sigemptyset(&sa.sa_mask);
+	sa.sa_flags = SA_RESTART; /* require BSD signal semantics */
+	sa.sa_handler = SIG_IGN;
+	sigaction(SIGINT, &sa, NULL);
+
+	if (interactive) {
+		while ((wpid = wait(NULL)) >= 0 && wpid != pid)
+			;
+		updatesidebar();
+		updatetitle();
+		init();
+	} else {
+		sa.sa_handler = sighandler;
+		sigaction(SIGINT, &sa, NULL);
+	}
+}
+
 /* Pipe item line or item field to a program.
    If `field` is -1 then pipe the TSV line, else a specified field.
-   if `wantoutput` is 1 then cleanup and restore the tty,
+   if `interactive` is 1 then cleanup and restore the tty and wait on the
+   process.
    if 0 then don't do that and also write stdout and stderr to /dev/null. */
 void
-pipeitem(const char *cmd, struct item *item, int field, int wantoutput)
+pipeitem(const char *cmd, struct item *item, int field, int interactive)
 {
 	FILE *fp;
-	int i, pid, wpid, status;
+	pid_t pid;
+	int i, status;
 
-	if (wantoutput)
+	if (interactive)
 		cleanup();
 
 	switch ((pid = fork())) {
 	case -1:
 		die("fork");
 	case 0:
-		if (!wantoutput) {
+		if (!interactive) {
 			dup2(devnullfd, 1);
 			dup2(devnullfd, 2);
 		}
@@ -593,28 +626,30 @@ pipeitem(const char *cmd, struct item *item, int field, int wantoutput)
 		status = WIFEXITED(status) ? WEXITSTATUS(status) : 127;
 		_exit(status);
 	default:
-		while ((wpid = wait(NULL)) >= 0 && wpid != pid)
-			;
-
-		if (wantoutput) {
-			updatesidebar(onlynew);
-			updatetitle();
-			init();
-		}
+		processexit(pid, interactive);
 	}
 }
 
 void
-forkexec(char *argv[])
+forkexec(char *argv[], int interactive)
 {
-	switch (fork()) {
+	pid_t pid;
+
+	if (interactive)
+		cleanup();
+
+	switch ((pid = fork())) {
 	case -1:
 		die("fork");
 	case 0:
-		dup2(devnullfd, 1);
-		dup2(devnullfd, 2);
+		if (!interactive) {
+			dup2(devnullfd, 1);
+			dup2(devnullfd, 2);
+		}
 		if (execvp(argv[0], argv) == -1)
 			_exit(1);
+	default:
+		processexit(pid, interactive);
 	}
 }
 
@@ -626,8 +661,7 @@ pane_row_get(struct pane *p, off_t pos)
 
 	if (p->row_get)
 		return p->row_get(p, pos);
-	else
-		return p->rows + pos;
+	return p->rows + pos;
 }
 
 char *
@@ -636,8 +670,7 @@ pane_row_text(struct pane *p, struct row *row)
 	/* custom formatter */
 	if (p->row_format)
 		return p->row_format(p, row);
-	else
-		return row->text;
+	return row->text;
 }
 
 int
@@ -776,7 +809,7 @@ cyclepanen(int n)
 void
 cyclepane(void)
 {
-	int i;
+	size_t i;
 
 	i = selpane;
 	cyclepanen(+1);
@@ -790,6 +823,9 @@ updategeom(void)
 {
 	int w, x;
 
+	panes[PaneFeeds].width = getsidebarwidth();
+	if (win.width && panes[PaneFeeds].width > win.width)
+		panes[PaneFeeds].width = win.width - 1;
 	panes[PaneFeeds].x = 0;
 	panes[PaneFeeds].y = 0;
 	/* reserve space for statusbar */
@@ -821,6 +857,7 @@ updategeom(void)
 	scrollbars[PaneItems].x = panes[PaneItems].x + panes[PaneItems].width;
 	scrollbars[PaneItems].y = panes[PaneItems].y;
 	scrollbars[PaneItems].size = panes[PaneItems].height;
+	scrollbars[PaneItems].hidden = panes[PaneItems].width ? 0 : 1;
 
 	/* statusbar below */
 	statusbar.width = win.width;
@@ -982,12 +1019,10 @@ char *
 uiprompt(int x, int y, char *fmt, ...)
 {
 	va_list ap;
-	char *input;
-	char buf[32];
+	char *input, buf[32];
 
 	va_start(ap, fmt);
-	if (vsnprintf(buf, sizeof(buf), fmt, ap) >= sizeof(buf))
-		buf[sizeof(buf) - 1] = '\0';
+	vsnprintf(buf, sizeof(buf), fmt, ap);
 	va_end(ap);
 
 	cursorsave();
@@ -1300,7 +1335,7 @@ feeds_reloadall(void)
 	urls_free();
 	/* restore numeric position */
 	pane_setpos(&panes[PaneItems], pos);
-	updatesidebar(onlynew);
+	updatesidebar();
 	updatetitle();
 }
 
@@ -1308,7 +1343,8 @@ int
 getsidebarwidth(void)
 {
 	struct feed *feed;
-	int i, len, width = 0;
+	size_t i;
+	int len, width = 0;
 
 	for (i = 0; i < nfeeds; i++) {
 		feed = &feeds[i];
@@ -1326,7 +1362,7 @@ getsidebarwidth(void)
 }
 
 void
-updatesidebar(int onlynew)
+updatesidebar(void)
 {
 	struct pane *p;
 	struct row *row;
@@ -1461,7 +1497,7 @@ mousereport(int button, int release, int x, int y)
 			if (!p->nrows || pos >= p->nrows)
 				break;
 			pane_setpos(p, pos);
-			if (selpane == PaneFeeds) {
+			if (i == PaneFeeds) {
 				row = pane_row_get(p, p->pos);
 				f = (struct feed *)row->data;
 				feeds_set(f);
@@ -1470,14 +1506,14 @@ mousereport(int button, int release, int x, int y)
 					feed_load(f, f->fp);
 				urls_free();
 				/* redraw row: counts could be changed */
-				updatesidebar(onlynew);
+				updatesidebar();
 				updatetitle();
-			} else if (selpane == PaneItems) {
+			} else if (i == PaneItems) {
 				if (dblclick && !changedpane) {
 					row = pane_row_get(p, p->pos);
 					item = (struct item *)row->data;
 					markread(p, p->pos, p->pos, 1);
-					forkexec((char *[]) { plumbercmd, item->fields[FieldLink], NULL });
+					forkexec((char *[]) { plumbercmd, item->fields[FieldLink], NULL }, plumberia);
 				}
 			}
 			break;
@@ -1485,7 +1521,7 @@ mousereport(int button, int release, int x, int y)
 			if (!p->nrows || pos >= p->nrows)
 				break;
 			pane_setpos(p, pos);
-			if (selpane == PaneItems) {
+			if (i == PaneItems) {
 				row = pane_row_get(p, p->pos);
 				item = (struct item *)row->data;
 				markread(p, p->pos, p->pos, 1);
@@ -1507,13 +1543,19 @@ feed_row_format(struct pane *p, struct row *row)
 	struct feed *feed;
 	static char text[1024];
 	char bufw[256], counts[128];
-	int len;
+	int len, w;
 
 	feed = (struct feed *)row->data;
 
+	/* align counts to the right and pad remaining width with spaces */
 	len = snprintf(counts, sizeof(counts), "(%lu/%lu)",
 	               feed->totalnew, feed->total);
-	if (utf8pad(bufw, sizeof(bufw), feed->name, p->width - len, ' ') != -1)
+	if (len > p->width)
+		w = p->width;
+	else
+		w = p->width - len;
+
+	if (utf8pad(bufw, sizeof(bufw), feed->name, w, ' ') != -1)
 		snprintf(text, sizeof(text), "%s%s", bufw, counts);
 	else
 		text[0] = '\0';
@@ -1649,7 +1691,7 @@ markread(struct pane *p, off_t from, off_t to, int isread)
 			if (i >= visstart && i < visstart + p->height)
 				pane_row_draw(p, i, i == p->pos);
 		}
-		updatesidebar(onlynew);
+		updatesidebar();
 		updatetitle();
 	}
 }
@@ -1730,6 +1772,12 @@ main(int argc, char *argv[])
 		pipercmd = tmp;
 	if ((tmp = getenv("SFEED_YANKER")))
 		yankercmd = tmp;
+	if ((tmp = getenv("SFEED_PLUMBER_INTERACTIVE")))
+		plumberia = !strcmp(tmp, "1");
+	if ((tmp = getenv("SFEED_PIPER_INTERACTIVE")))
+		piperia = !strcmp(tmp, "1");
+	if ((tmp = getenv("SFEED_YANKER_INTERACTIVE")))
+		yankeria = !strcmp(tmp, "1");
 	if ((tmp = getenv("SFEED_MARK_READ")))
 		markreadcmd = tmp;
 	if ((tmp = getenv("SFEED_MARK_UNREAD")))
@@ -1785,7 +1833,7 @@ main(int argc, char *argv[])
 	if ((devnullfd = open("/dev/null", O_WRONLY)) == -1)
 		die("open: /dev/null");
 
-	updatesidebar(onlynew);
+	updatesidebar();
 	updatetitle();
 	init();
 	draw();
@@ -1942,7 +1990,7 @@ nextpage:
 				p = &panes[selpane];
 				row = pane_row_get(p, p->pos);
 				item = (struct item *)row->data;
-				forkexec((char *[]) { plumbercmd, item->fields[FieldEnclosure], NULL });
+				forkexec((char *[]) { plumbercmd, item->fields[FieldEnclosure], NULL }, plumberia);
 			}
 			break;
 		case 'm': /* toggle mouse mode */
@@ -1958,7 +2006,7 @@ nextpage:
 		case 't': /* toggle showing only new in sidebar */
 			onlynew = !onlynew;
 			pane_setpos(&panes[PaneFeeds], 0);
-			updatesidebar(onlynew);
+			updatesidebar();
 			break;
 		case 'o': /* feeds: load, items: plumb url */
 		case '\n':
@@ -1972,13 +2020,13 @@ nextpage:
 					feed_load(f, f->fp);
 				urls_free();
 				/* redraw row: counts could be changed */
-				updatesidebar(onlynew);
+				updatesidebar();
 				updatetitle();
 			} else if (selpane == PaneItems && panes[selpane].nrows) {
 				row = pane_row_get(p, p->pos);
 				item = (struct item *)row->data;
 				markread(p, p->pos, p->pos, 1);
-				forkexec((char *[]) { plumbercmd, item->fields[FieldLink], NULL });
+				forkexec((char *[]) { plumbercmd, item->fields[FieldLink], NULL }, plumberia);
 			}
 			break;
 		case 'c': /* items: pipe TSV line to program */
@@ -1991,11 +2039,11 @@ nextpage:
 				row = pane_row_get(p, p->pos);
 				item = (struct item *)row->data;
 				switch (ch) {
-				case 'y': pipeitem(yankercmd, item, FieldLink, 0); break;
-				case 'E': pipeitem(yankercmd, item, FieldEnclosure, 0); break;
+				case 'y': pipeitem(yankercmd, item, FieldLink, yankeria); break;
+				case 'E': pipeitem(yankercmd, item, FieldEnclosure, yankeria); break;
 				default:
 					markread(p, p->pos, p->pos, 1);
-					pipeitem(pipercmd, item, -1, 1);
+					pipeitem(pipercmd, item, -1, piperia);
 					break;
 				}
 			}
